*********************************************************
*** See Call32.txt for info on how to use the Call32 functions

***********************************************************************
PROCEDURE reg32
******************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427,2363@compuserve.com
***  Modified: 08/22/95
***  Function: Registers a 32 bit DLL function using CALL32.DLL
***            Thunk interface
***    Assume: Foxtools is loaded. CALL32.DLL is available
***      Pass: pcDLLFunction  -   Name of 32 bit function
***            pcDLLName      -   The DLL it is contained in
***            pcParmTypes    -   Parameter types
***                               types: "i"   - Integer (long)
***                                      "w"   - Window Handles
***                                      "p"   - Pointers (all strings)
***
***    Return: Function handle that must be used to CALL32 function
***
***    Partial Docs for Call32:
***
***    1. Summary
***    ----------
***
***    CALL32.DLL is a DLL that can be used for calling routines in 32-bit
***    DLLs on Windows NT/95.  Using it, a Visual Basic program, running
***    in the Win16 subsystem, can declare and call functions in any 32-bit
***    DLL (including, but not limited to, the system DLLs).  CALL32.DLL works
***    on both the x86 and MIPS versions on NT.  It has not been tested
***    on Alpha or other versions, but should work.
***
***
***    2. Usage
***    --------
***
***    To call a function in a 32-bit DLL, follow the following steps.  Declare
***    the "Declare32" function as follows (all one one line):
***
***    Declare Function Declare32 Lib "call32.dll" (ByVal Func$,
***           ByVal Library$, ByVal Args$) As Long
***
***    Next, declare the function you wish to call.  Declare it in the ordinary
***    fashion, with the following exceptions:
***       - Use a library name of "call32.dll"
***       - Use an Alias of "Call32"
***       - Add an additional argument at the end, of type ByVal Long
***    For example, if you are calling the function:
***      GetWindowText(HWND hwnd, LPSTR lpsz, int cch)
***    declare it as follows (remember that ints and all handles are 32 bits,
***    so use a Long):
***      Declare Function GetWindowText Lib "call32.dll" Alias "Call32"
***            (ByVal hwnd As Long, ByVal lpsz As String,
***             ByVal cch As Long, ByVal id As Long) As Long
***
***    In the initialization section of your application, you declare the
***    actual library and name of the function you want to call with
***    the Declare32 function.  Pass it the name of the function, the
***    library, and a string describing the argument types.  Each letter
***    in the string declares the type of one argument, and should be
***    either "i" for a 32 bit integer or handle type, "p" for any
***    pointer type, or "w" for an HWND parameter you want to pass
***    a 16 bit HWND to and have be automatically converted to a 32 bit
***    HWND.  The return value of Declare32 should be saved away in
***    a global variable to be passed as the last parameter to the
***    function you declared earlier.  So, continue the example, you
***    would call:
***
***      idGetWindowText = Declare32("GetWindowText", "user32", "wpi")
***
***    (As a side note, this would be more properly declared as
***    "GetWindowTextA", since this is the real exported name.  However,
***    Declare32 will automatically add an "A" to the end of a
***    function name if necessary).
***
***    To call the function, you would just call:
***
***      cbCopy = GetWindowText(hwnd, sz, cb, idGetWindowText)
***
***  FoxPro Example:
***     lnCall32=Reg32("GetVersion","Kernel32.dll","")
***
***     *** Alternately, this is the manual approach:
***     *** lhCall32=regfn("Declare32","CCC","L","CALL32.DLL")
***     *** lnCall32=callfn(lhCall32,"GetVersion","Kernel32.DLL","")
***     *** ? lnCall32
***
***     *** Note the extra Long parameter even though no parm
***     *** is required by the API function!
***     *** You must always call CALL32!!! The long parm
***     *** passed last actually routes the call to the
***     *** proper 32 bit function!
***
***
***     lhWinVersion=RegFP("","L")
***     *** Alternately you can use this manual approach:
***     *** lhWinVersion=regfn("Call32","L","L","CALL32.DLL")
***     lnVersion=callfn(lhWinVersion,lnCall32)
***
************************************************************************
PARAMETERS pcdllfunction,pcdllname,pcparmtypes
PRIVATE lhcall32,lncall32

lhcall32=regfn("Declare32","CCC","L","CALL32.DLL")
lncall32=callfn(lhcall32,pcdllfunction,pcdllname,pcparmtypes)

RETURN lncall32

************************************************************************
PROCEDURE regfp
******************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427,2363@compuserve.com
***  Modified: 08/22/95
***  Function: Wrapper for Call32 function, allows direct call
***            of 32 bit DLLs passing parameters as String!
***    Assume: FoxTools loaded, CALL32.DLL available
***      Pass: pcParms      -   Parameter types (like RegFn())
***            pcRetType    -   Return Type (like RegFn()))
***    Return: Function Return value
************************************************************************
PARAMETER pcparms,pcrettype

RETURN regfn("Call32",pcparms+"L",pcrettype,"CALL32.DLL")


************************************************************************
PROCEDURE W32Version
******************
***  Modified: 01/06/96
***  Function: Returns Windows Version Number (multiplied by 100)
*************************************************************************
lhw32ver=regfn("W32Version","","I","call32.dll")
RETURN callfn(lhw32ver)


************************************************************************
PROCEDURE rdregstr
******************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427,2363@compuserve.com
***  Modified: 08/24/95
***  Function: Reads a single String value from the Registry
***    Assume: Requires Call32.dll
***            Probably works on Binary entries as well
***      Pass: pcRoot  -   string of registry key values
***                        "HKEY_CLASSES_ROOT"
***                        "HKEY_CURRENT_USER"
***                        "HKEY_LOCAL_MACHINE"
***                        "HKEY_USERS"
***            pcSubKey-   Subkey 'path' ("SOFTWARE\Windows")
***            pcValue -   Actual key to read
***    Return: key value, "", "*ERROR*"
************************************************************************
PARAMETERS pcroot,pcsubkey,pckey,pnlength
PRIVATE lcresult,lnlength,lnresult

IF PARAMETERS()<3 OR ;
		TYPE("pcRoot")#"C" OR ;
		TYPE("pcSubKey")#"C" OR ;
		TYPE("pcKey")#"C"
	RETURN "*ERROR*"
ENDIF

pnlength=IIF(TYPE("pnLength")="N",pnlength,2048)

lcresult=SPACE(pnlength)

*** long PASCAL _export ReadRegistry
*** (LPCSTR cHKey,LPSTR Subkey,LPSTR Value,LPSTR Result,INT Length)

lhreg=regfn("ReadRegistry","C@C@C@CI","L","call32.dll")
lnresult=callfn(lhreg,lcroot,@pcsubkey,@pckey,@lcresult,pnlength)

*** Return only left of Null
lcresult=TRIM(LEFT(lcresult,AT(CHR(0),lcresult)-1))

IF lnresult#0
	RETURN "*ERROR*"
ENDIF

RETURN lcresult
*EOP RegString

************************************************************************
PROCEDURE rdregint
******************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427,2363@compuserve.com
***  Modified: 08/24/95
***  Function: Reads a single String value from the Registry
***    Assume: Requires Call32.dll
***      Pass: pcRoot  -   string of registry key values
***                        "HKEY_CLASSES_ROOT"
***                        "HKEY_CURRENT_USER"
***                        "HKEY_LOCAL_MACHINE"
***                        "HKEY_USERS"
***            pcSubKey-   Subkey 'path' ("SOFTWARE\Windows")
***            pcValue -   Actual key to read
***            pnLength-   Byte length (16)
***    Return: key value or -1 0 if empty or non-existant
************************************************************************
PARAMETERS pcroot,pcsubkey,pckey,pnlength
PRIVATE lcresult,lnlength,lnresult

IF PARAMETERS()<3 OR ;
		TYPE("pcRoot")#"C" OR ;
		TYPE("pcSubKey")#"C" OR ;
		TYPE("pcKey")#"C"
	RETURN -1
ENDIF

pnlength=IIF(TYPE("pnLength")="N",pnlength,16)

lnresult=0
lnlength=16

*** long PASCAL _export ReadRegistry
*** (LPCSTR cHKey,LPCSTR Subkey,LPCSTR Key,LPSTR Result,DWORD Length)
*** Note here the pointer to receive the value is changed to a Long
*** in the RegFn() declaration.

lhreg=regfn("ReadRegistry","C@C@C@LI","L","call32.dll")
lnretval=callfn(lhreg,lcroot,@pcsubkey,@pckey,@lnresult,pnlength)

IF lnretval#0
	RETURN -1
ENDIF

RETURN lnresult
*EOP RdRegInt


************************************************************************
PROCEDURE openkey
******************
***  Modified: 12/08/95
***  Function: Opens a registry key before reading
***            writing entries.
***    Assume: Calls 16 bit Regopen in CALL32.DLL
***             because of limitations in Longs & HKEY
***      Pass: tcHkey   -  "HKEY_..." strings
***            tcSubkey -  Reg path "\Software\version"
***    Return: key handle
*************************************************************************
PARAMETER lchkey,lcsubkey
lnrethandle=0
lhropen=regfn("RegOpen","CC","L","CALL32.DLL")
lnkey=callfn(lhropen,lchkey,lcsubkey)
RETURN lnkey

************************************************************************
PROCEDURE closekey
******************
***  Function: Close registry key.
***      Pass: thHandle - Key handle
*************************************************************************
PARAMETER thhandle
lhclose=regfn("RegCloseKey","L","L","shell.dll")
RETURN callfn(lhclose,thhandle)

************************************************************************
FUNCTION querystr
******************
***  Modified: 12/07/95
***  Function: Reads a registry string
***     Notes: Also works with Binary types
***            as long as it doesn't contain
***            NULL values.
***      Pass: tnHandle - Key Handle
***            tcEntry  - Entry to retrieve
***    Return: string or "" if empty or "*ERROR*"
************************************************************************
PARAMETER tnhandle,tcentry,tlbinary
PRIVATE lhcall32,lhfp,lcdatabuffer

lhcall32=reg32("RegQueryValueEx","ADVAPI32.dll","ipippp")
lhfp=regfp("LCL@L@C@L","L")

*** Return buffer to receive value
lcdatabuffer=SPACE(1024)
lnsize=LEN(lcdatabuffer)

lntype=1         && REG_SZ

lnresult=callfn(lhfp,tnhandle,tcentry,0,@lntype,;
	@lcdatabuffer,@lnsize,lhcall32)
IF lnresult#0
	RETURN "*ERROR*"
ENDIF

IF lnsize<2
	RETURN ""
ENDIF

*** Return string and strip out NULLs
RETURN SUBSTR(CHRTRAN(lcdatabuffer,CHR(0),""),1,lnsize-1)
* ReadRegistryString

************************************************************************
FUNCTION queryint
******************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427.2363@compuserve.com
***  Modified: 12/07/95
***  Function: Reads a registry string
***      Pass: tnHandle - Key Handle
***            tcEntry  - Entry to retrieve
***    Return: string or "" if empty or "*ERROR*"
************************************************************************
PARAMETER tnhandle,tcentry
PRIVATE lhcall32,lhfp,lcdatabuffer

lhcall32=reg32("RegQueryValueEx","ADVAPI32.dll","ipippp")
lhfp=regfp("LCL@L@L@L","L")

*** Return buffer to receive value
lndatabuffer=0
lnsize=8
lntype=1         && REG_SZ

lnresult=callfn(lhfp,tnhandle,tcentry,0,@lntype,;
	@lndatabuffer,@lnsize,lhcall32)

IF lnresult#0
	RETURN -1
ENDIF

RETURN lndatabuffer
* ReadRegistryString

************************************************************************
PROCEDURE writestr
******************
***  Modified: 12/08/95
***  Function: Writes a key value to an entry.
***    Assume: Uses Call32.dll
***      Pass: tnHandle - Key Handle
***            tcEntry  - The entry off the key
***            tcValue  - Value to set it to
***    Return: .T. or .F.
*************************************************************************
PARAMETERS tnhandle, tcentry, tcvalue
PRIVATE  lnsize, lcdatabuffer, tntype

lhreg32=reg32("RegSetValueEx","advapi32.dll","ipiipi")
lhregfp=regfp("LCLLCL","L")

lnsize=LEN(tcvalue)
lntype=1   && REG_SZ
lnresult=callfn(lhregfp,tnhandle,tcentry,0,1,;
	tcvalue,lnsize,lhreg32)

IF lnresult#0
	RETURN .F.
ENDIF

RETURN .T.
* WriteStr

************************************************************************
PROCEDURE writeint
******************
***  Modified: 12/08/95
***  Function: Writes a key value to an entry.
***    Assume: Uses Call32.dll
***      Pass: tnHandle - Key Handle
***            tcEntry  - The entry off the key
***            tnValue  - Value to set it to
***    Return: .T. or .F.
*************************************************************************
PARAMETERS tnhandle, tcntry, tnvalue
PRIVATE  lnsize, lcdatabuffer, lntype

lhreg32=reg32("RegSetValueEx","advapi32.dll","ipiiii")
lhregfp=regfp("LCLLLL","L")

lnsize=8
lntype=4   && REG_DWORD

lnresult=callfn(lhregfp,tnhandle,tcentry,0,lntype,;
	tnvalue,lnsize,lhreg32)

IF lnresult#0
	RETURN -1
ENDIF

RETURN tnvalue
* WriteStr



